import {
  AD_REPOSITORY,
  INPUT_DATETIME_TRANSFORMER,
} from '@modules/ad/ad.di-tokens';
import { AdUpdatedDomainEvent } from '@modules/ad/core/domain/events/ad.domain-event';
import { Inject } from '@nestjs/common';
import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { EventEmitter2 } from '@nestjs/event-emitter';
import { AdRepositoryPort } from '../../ports/ad.repository.port';
import { DateTimeTransformerPort } from '../../ports/datetime-transformer.port';
import { createPropsFromCommand } from '../create-ad/create-ad.service';
import { UpdateAdCommand } from './update-ad.command';

@CommandHandler(UpdateAdCommand)
export class UpdateAdService implements ICommandHandler {
  constructor(
    @Inject(AD_REPOSITORY)
    private readonly repository: AdRepositoryPort,
    @Inject(INPUT_DATETIME_TRANSFORMER)
    private readonly datetimeTransformer: DateTimeTransformerPort,
    private readonly eventEmitter: EventEmitter2,
  ) {}

  async execute(command: UpdateAdCommand): Promise<void> {
    const ad = await this.repository.findOneById(command.adId, {
      waypoints: true,
      schedule: true,
    });
    ad.update(createPropsFromCommand(command, this.datetimeTransformer));
    await this.repository.update(ad.id, ad);
    this.eventEmitter.emitAsync(
      AdUpdatedDomainEvent.name,
      new AdUpdatedDomainEvent(ad),
    );
  }
}
