import {
  NotFoundException,
  RpcExceptionCode,
  RpcValidationPipe,
} from '@mobicoop/ddd-library';
import { UpdateAdCommand } from '@modules/ad/core/application/commands/update-ad/update-ad.command';
import { Controller, UsePipes } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { GRPC_SERVICE_NAME } from '@src/app.constants';
import { UpdateAdRequestDto } from './dtos/update-ad.request.dto';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class UpdateAdGrpcController {
  constructor(private readonly commandBus: CommandBus) {}

  @GrpcMethod(GRPC_SERVICE_NAME, 'Update')
  async update(data: UpdateAdRequestDto): Promise<void> {
    try {
      const cmdProps = {
        adId: data.id,
        ...data,
      };
      delete (cmdProps as { id?: string }).id;

      await this.commandBus.execute(new UpdateAdCommand(cmdProps));
    } catch (error) {
      if (error instanceof NotFoundException) {
        throw new RpcException({
          code: RpcExceptionCode.NOT_FOUND,
          message: error.message,
        });
      }
      throw error;
    }
  }
}
